/*
Copyright (c) 2014, Pure Engineering LLC
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>	
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>
#include <limits.h>
//add lib
#include <wiringPi.h>
#include <wiringPiI2C.h>
 
#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))

//Pin difine for LCD
#define PIN_RST 7
#define PIN_CS  0
#define PIN_D7  2
#define PIN_D6  3
#define PIN_D5 30
#define PIN_D4 21
#define PIN_D3 22
#define PIN_D2 23
#define PIN_D1 24
#define PIN_D0 25
#define PIN_WR 28
#define PIN_RS 29
#define PIN_LED 27

//Color codes
#define RED  	0xf800
#define GREEN	0x07e0
#define BLUE 	0x001f
#define WHITE	0xffff
#define BLACK	0x0000
#define YELLOW   0xFFE0

//Configuration for LCD
#define VERTICAL 0
#define HORIZONTAL 1
#define LCD_ALIGNMENT VERTICAL

//ILI932x registers
#define ILI932X_START_OSC          0x00
#define ILI932X_DRIV_OUT_CTRL      0x01
#define ILI932X_DRIV_WAV_CTRL      0x02
#define ILI932X_ENTRY_MOD          0x03
#define ILI932X_RESIZE_CTRL        0x04
#define ILI932X_DISP_CTRL1         0x07
#define ILI932X_DISP_CTRL2         0x08
#define ILI932X_DISP_CTRL3         0x09
#define ILI932X_DISP_CTRL4         0x0A
#define ILI932X_RGB_DISP_IF_CTRL1  0x0C
#define ILI932X_FRM_MARKER_POS     0x0D
#define ILI932X_RGB_DISP_IF_CTRL2  0x0F
#define ILI932X_POW_CTRL1          0x10
#define ILI932X_POW_CTRL2          0x11
#define ILI932X_POW_CTRL3          0x12
#define ILI932X_POW_CTRL4          0x13
#define ILI932X_GRAM_HOR_AD        0x20
#define ILI932X_GRAM_VER_AD        0x21
#define ILI932X_RW_GRAM            0x22
#define ILI932X_POW_CTRL7          0x29
#define ILI932X_FRM_RATE_COL_CTRL  0x2B
#define ILI932X_GAMMA_CTRL1        0x30
#define ILI932X_GAMMA_CTRL2        0x31
#define ILI932X_GAMMA_CTRL3        0x32
#define ILI932X_GAMMA_CTRL4        0x35
#define ILI932X_GAMMA_CTRL5        0x36
#define ILI932X_GAMMA_CTRL6        0x37
#define ILI932X_GAMMA_CTRL7        0x38
#define ILI932X_GAMMA_CTRL8        0x39
#define ILI932X_GAMMA_CTRL9        0x3C
#define ILI932X_GAMMA_CTRL10       0x3D
#define ILI932X_HOR_START_AD       0x50
#define ILI932X_HOR_END_AD         0x51
#define ILI932X_VER_START_AD       0x52
#define ILI932X_VER_END_AD         0x53
#define ILI932X_GATE_SCAN_CTRL1    0x60
#define ILI932X_GATE_SCAN_CTRL2    0x61
#define ILI932X_GATE_SCAN_CTRL3    0x6A
#define ILI932X_PART_IMG1_DISP_POS 0x80
#define ILI932X_PART_IMG1_START_AD 0x81
#define ILI932X_PART_IMG1_END_AD   0x82
#define ILI932X_PART_IMG2_DISP_POS 0x83
#define ILI932X_PART_IMG2_START_AD 0x84
#define ILI932X_PART_IMG2_END_AD   0x85
#define ILI932X_PANEL_IF_CTRL1     0x90
#define ILI932X_PANEL_IF_CTRL2     0x92
#define ILI932X_PANEL_IF_CTRL3     0x93
#define ILI932X_PANEL_IF_CTRL4     0x95
#define ILI932X_PANEL_IF_CTRL5     0x97
#define ILI932X_PANEL_IF_CTRL6     0x98


//define for Lepton
#define ID 0x2a //I2C address
#define NCOLORS 1280 // Colors
#define T_C 30  // temp
#define N 256
#define LEPTON_WRITE (0x54)
#define LEPTON_READ  (0x55)
#define LEPTON_WRITEWAIT (0)
#define LEPTON_AGC (0x01)
#define LEPTON_SYS (0x02)
#define LEPTON_VID (0x03)
#define LEPTON_GET (0x00)
#define LEPTON_SET (0x01)
#define LEPTON_RUN (0x02)
#define LEPTON_NACK (0)
#define LEPTON_ACK (1)


unsigned int colors[1280];
unsigned int col[3][1280];
int fd, fdi;
static const char *device = "/dev/spidev0.0";
static uint8_t mode;
static uint8_t bits = 8;
static uint32_t speed = 16000000;
static uint16_t delayu;




#define VOSPI_FRAME_SIZE (164)
uint8_t lepton_frame_packet[VOSPI_FRAME_SIZE];
static unsigned int lepton_image[80][80];
static unsigned int lepton_image_scale[80][80];
int tempmin,tempmax ;
float tempminf,tempmaxf ;

    char s1[N] = {'\0'};
    char s2[N] = {'\0'};




// ******************************
//  function
// ******************************

//GPIO Initialize
static void initgpio()
{	
	if(wiringPiSetup() == -1){
    printf("error wiringPi setup\n");
    //return 1;
    }

    pinMode(PIN_RST, OUTPUT);
    pinMode(PIN_CS, OUTPUT);
    pinMode(PIN_D7, OUTPUT);
    pinMode(PIN_D6, OUTPUT);
    pinMode(PIN_D5, OUTPUT);
    pinMode(PIN_D4, OUTPUT);
    pinMode(PIN_D3, OUTPUT);
    pinMode(PIN_D2, OUTPUT);
    pinMode(PIN_D1, OUTPUT);
    pinMode(PIN_D0, OUTPUT);
    pinMode(PIN_WR, OUTPUT);
    pinMode(PIN_RS, OUTPUT);
    pinMode(PIN_LED, OUTPUT);
    digitalWrite(PIN_LED, 1);
    
}

//LCD out
static void LCD_busout(unsigned char data)
{
	unsigned char d0,d1,d2,d3,d4,d5,d6,d7 ;
	
d0 = data & 0x01;
d1 = (data >> 1) & 0x01;
d2 = (data >> 2) & 0x01;
d3 = (data >> 3) & 0x01;
d4 = (data >> 4) & 0x01;
d5 = (data >> 5) & 0x01;
d6 = (data >> 6) & 0x01;
d7 = (data >> 7) & 0x01;
	
  digitalWrite(PIN_D0, d0 );
  digitalWrite(PIN_D1, d1 );
  digitalWrite(PIN_D2, d2 );
  digitalWrite(PIN_D3, d3 );
  digitalWrite(PIN_D4, d4 );
  digitalWrite(PIN_D5, d5 );
  digitalWrite(PIN_D6, d6 );
  digitalWrite(PIN_D7, d7 );

}

static void LCD_WriteIndex(unsigned int index)
{
  // Write Index
  digitalWrite(PIN_RS, 0);
  LCD_busout(0);
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);
  LCD_busout(index);
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);
  
}

static void LCD_WriteData(unsigned int data)
{
  // Write Data
  digitalWrite(PIN_RS, 1);
  LCD_busout((data >> 8) & 0xFF);  
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);
  LCD_busout( data & 0xFF);  
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);
}  


static void LCD_WriteReg(unsigned int index, unsigned int data)
{
  // Write Index
  LCD_WriteIndex(index);
  // Write Data
  LCD_WriteData(data);
}  

static void LCD_SolidFill(unsigned long count, unsigned int color)
{
  // Write Index
  LCD_WriteIndex(0x22);
  
  // Write Data
  	digitalWrite(PIN_RS, 1);

  LCD_busout((color >> 8) & 0xFF);
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);
  
  LCD_busout( color & 0xFF);
  digitalWrite(PIN_WR, 0);
  digitalWrite(PIN_WR, 1);

unsigned long i;

  for (i=0; i <= count-1; i++){
    // Write Data
    LCD_busout((color >> 8) & 0xFF);
    digitalWrite(PIN_WR, 0);
    digitalWrite(PIN_WR, 1);
    LCD_busout(color & 0xFF);
    digitalWrite(PIN_WR, 0);
    digitalWrite(PIN_WR, 1);
    }
}

static void LCD_SetCursor(unsigned int x, unsigned int y)
{
if (x < 240 && y < 320) {
  #if LCD_ALIGNMENT == VERTICAL
    LCD_WriteReg(0x20, x);
    LCD_WriteReg(0x21, y);
  #elif LCD_ALIGNMENT == HORIZONTAL 
    LCD_WriteReg(0x20, 240-y);
    LCD_WriteReg(0x21, x);
  #endif  
  }
}

static void LCD_Clear(unsigned int color)
{
  LCD_SetCursor(0,0);
  LCD_SolidFill(76800, color);  // This is a faster method
}

static void resetlcd()
{
	digitalWrite(PIN_CS, 1);
	digitalWrite(PIN_RST, 0);
	digitalWrite(PIN_RS, 0);
	digitalWrite(PIN_WR, 1);
	delay(20) ;
	digitalWrite(PIN_RST, 0);
	delay(20) ;
	digitalWrite(PIN_RST, 1);
	digitalWrite(PIN_CS,0);
	digitalWrite(PIN_RS, 0);
	delay(20) ;
	LCD_busout(0);	
    digitalWrite(PIN_WR, 0);
  	delay(20) ;
    digitalWrite(PIN_WR, 1);
	delay(20) ;
	LCD_busout(0);
    digitalWrite(PIN_WR, 0);
  	delay(20) ;
    digitalWrite(PIN_WR, 1);
  	delay(20) ;
	LCD_busout(0);
    digitalWrite(PIN_WR, 0);
  	delay(20) ;
    digitalWrite(PIN_WR, 1);
	LCD_busout(0);
	digitalWrite(PIN_RS, 1);
	delay(20) ;
  	digitalWrite(PIN_CS, 1);
	delay(20) ;
}

static void initlcd()
{
	resetlcd();
	digitalWrite(PIN_CS,0);
	
	LCD_WriteReg(ILI932X_START_OSC        , 0x0001); // Start oscillator
	delay(50);     // 50 millisecond delay
	LCD_WriteReg(ILI932X_DRIV_OUT_CTRL    , 0x0100);
	LCD_WriteReg(ILI932X_DRIV_WAV_CTRL    , 0x0700);
	LCD_WriteReg(ILI932X_ENTRY_MOD        , 0x1038);
	LCD_WriteReg(ILI932X_RESIZE_CTRL      , 0x0000);
	LCD_WriteReg(ILI932X_DISP_CTRL2       , 0x0202);
	LCD_WriteReg(ILI932X_DISP_CTRL3       , 0x0000);
	LCD_WriteReg(ILI932X_DISP_CTRL4       , 0x0000);
	LCD_WriteReg(ILI932X_RGB_DISP_IF_CTRL1, 0x0);
	LCD_WriteReg(ILI932X_FRM_MARKER_POS   , 0x0);
	LCD_WriteReg(ILI932X_RGB_DISP_IF_CTRL2, 0x0);
	LCD_WriteReg(ILI932X_POW_CTRL1        , 0x0000);
	LCD_WriteReg(ILI932X_POW_CTRL2        , 0x0007);
	LCD_WriteReg(ILI932X_POW_CTRL3        , 0x0000);
	LCD_WriteReg(ILI932X_POW_CTRL4        , 0x0000);
	delay(200);
	LCD_WriteReg(ILI932X_POW_CTRL1        , 0x1690);
	LCD_WriteReg(ILI932X_POW_CTRL2        , 0x0227);
	delay(50);
	LCD_WriteReg(ILI932X_POW_CTRL3        , 0x001A);
	delay(50);
	LCD_WriteReg(ILI932X_POW_CTRL4        , 0x1800);
	LCD_WriteReg(ILI932X_POW_CTRL7        , 0x002A);
	delay(50);
	LCD_WriteReg(ILI932X_GAMMA_CTRL1      , 0x0000);
	LCD_WriteReg(ILI932X_GAMMA_CTRL2      , 0x0000);
	LCD_WriteReg(ILI932X_GAMMA_CTRL3      , 0x0000);
	LCD_WriteReg(ILI932X_GAMMA_CTRL4      , 0x0206);
	LCD_WriteReg(ILI932X_GAMMA_CTRL5      , 0x0808);
	LCD_WriteReg(ILI932X_GAMMA_CTRL6      , 0x0007);
	LCD_WriteReg(ILI932X_GAMMA_CTRL7      , 0x0201);
	LCD_WriteReg(ILI932X_GAMMA_CTRL8      , 0x0000);
	LCD_WriteReg(ILI932X_GAMMA_CTRL9      , 0x0000);
	LCD_WriteReg(ILI932X_GAMMA_CTRL10     , 0x0000);
	LCD_WriteReg(ILI932X_GRAM_HOR_AD      , 0x0000);
	LCD_WriteReg(ILI932X_GRAM_VER_AD      , 0x0000);
	LCD_WriteReg(ILI932X_HOR_START_AD     , 0x0000);
	LCD_WriteReg(ILI932X_HOR_END_AD       , 0x00EF);
	LCD_WriteReg(ILI932X_VER_START_AD     , 0X0000);
	LCD_WriteReg(ILI932X_VER_END_AD       , 0x013F);
	LCD_WriteReg(ILI932X_GATE_SCAN_CTRL1  , 0xA700); // Driver Output Control (R60h)
	LCD_WriteReg(ILI932X_GATE_SCAN_CTRL2  , 0x0003); // Driver Output Control (R61h)
	LCD_WriteReg(ILI932X_GATE_SCAN_CTRL3  , 0x0000); // Driver Output Control (R62h)
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL1   , 0X0010); // Panel Interface Control 1 (R90h)
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL2   , 0X0000);
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL3   , 0X0003);
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL4   , 0X1100);
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL5   , 0X0000);
	LCD_WriteReg(ILI932X_PANEL_IF_CTRL6   , 0X0000);
	LCD_WriteReg(ILI932X_DISP_CTRL1       , 0x0133); // Main screen turn on

	digitalWrite(PIN_CS, 1);  
	delay(30);
	digitalWrite(PIN_CS, 0);
}

static void LCD_ResetWindow()
{
  	LCD_WriteReg(0x50, 0x0000); //Horizontal Address Start Position  
  	LCD_WriteReg(0x51, 0x00EF); //Horizontal Address end Position (239)
  	LCD_WriteReg(0x52, 0x0000); //Vertical Address Start Position  
  	LCD_WriteReg(0x53, 0x013F); //Vertical Address end Position (319) 
}

static void LCD_SetWindow(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2)
{
  #if LCD_ALIGNMENT == VERTICAL  
    LCD_WriteReg(0x50, x1); //Horizontal Address Start Position  
    LCD_WriteReg(0x51, x2); //Horizontal Address end Position (239)  
    LCD_WriteReg(0x52, y1); //Vertical Address Start Position  
    LCD_WriteReg(0x53, y2); //Vertical Address end Position (319)                
  #elif LCD_ALIGNMENT == HORIZONTAL 
    #error "Windows Clearing in Horizontal mode is not possible yet!"
  #endif     
}

static void LCD_Rectangle(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, int color)
{      
    LCD_SetWindow(x1,y1,x2,y2);
    LCD_SetCursor(x1,y1);
    
    unsigned int width = x2-x1+1;
    unsigned int height = y2-y1+1;
    LCD_SolidFill(width*height, color);
    LCD_ResetWindow();
}

void lepton_set_reg(unsigned int reg)
{
   wiringPiI2CWrite (fdi,(reg >> 8) & 0xff  ) ;
   wiringPiI2CWrite (fdi, reg       & 0xff  ) ;
}

unsigned int lepton_read_reg(unsigned int reg)
{
	//
	// This function is not work now, still debuging....
	//
  	unsigned int reading  = 0;  
  	delay(LEPTON_WRITEWAIT);
  	reading = wiringPiI2CReadReg16 (fdi, reg) ;
  	return reading;
}


void lepton_command(unsigned int moduleID, unsigned int commandID, unsigned int command)
{
	//
	// This function is not work now, still debuging....
	//	
  	while (lepton_read_reg(0x2) & 0x01)
  	{
       delay(2000);
  	}
 
  	unsigned char  commandcode;
  
   	wiringPiI2CWrite (fdi,0x00  ) ;
  	delay(LEPTON_WRITEWAIT);
  	wiringPiI2CWrite (fdi,0x04  ) ;
    delay(LEPTON_WRITEWAIT);
    if (moduleID == 0x08) //OEM module ID
  	{
   		wiringPiI2CWrite (fdi,0x48  ) ;
	}
  	else
  	{
 		wiringPiI2CWrite (fdi,moduleID & 0x0f  ) ;
  	}
  	delay(LEPTON_WRITEWAIT);
  	commandcode = ((commandID << 2 ) & 0xfc) | (command & 0x3);
    wiringPiI2CWrite (fdi,commandcode  ) ;
  	delay(LEPTON_WRITEWAIT);
}

unsigned int lepton_read_data()
{
	//
	// This function is not work now, still debuging....
	//	

	int i ;
  	unsigned int data;
  	int payload_length;

	while (lepton_read_reg(0x2) & 0x01)
  	{
     	printf("busy\n\r");
  	}
	payload_length = lepton_read_reg(0x6);
  	printf("payload_length=%d\n\r",payload_length);
 
    for (i = 0; i < (payload_length / 2); i++)
  	{
   		data = wiringPiI2CRead (fdi) ;
   	}

 return data ;
 
}

void lepton_info()
{
	//
	// This function is not work now, still debuging....
	//	
    float auxtemp;
    unsigned int aux;
    float fpatemp;
    unsigned int fpa;
 	printf("SYS Aux Temperature Kelvin:\n");
	lepton_command(LEPTON_SYS, 0x10 >> 2 , LEPTON_GET);
	aux=lepton_read_data() ;
    auxtemp =( aux/ 100.0f) - 273.15 ;
	printf("SYS Fpa Temperature Kelvin:\n");
  	lepton_command(LEPTON_SYS, 0x14 >> 2 , LEPTON_GET);
  	fpa=lepton_read_data() ;
    fpatemp = fpa/ 100.0f-273.15;
    printf(" fpa temp: %f\n\r",fpatemp);
    printf(" aux temp: %f\n\r",auxtemp);
}

static void pabort(const char *s)
{
	perror(s);
	abort();
}

static void save_pgm_file(void)
{
	int i;
	int j;
	unsigned int maxval = 0;
	unsigned int minval = UINT_MAX;
	char image_name[32];
	int image_index = 0;

	do {
		sprintf(image_name, "IMG_%.4d.pgm", image_index);
		image_index += 1;
		if (image_index > 9999) 
		{
			image_index = 0;
			break;
		}

	} while (access(image_name, F_OK) == 0);

	FILE *f = fopen(image_name, "w");
	if (f == NULL)
	{
		printf("Error opening file!\n");
		exit(1);
	}

	printf("Calculating min/max values for proper scaling...\n");
	for(i=0;i<60;i++)
	{
		for(j=0;j<80;j++)
		{
			if (lepton_image[i][j] > maxval) {
				maxval = lepton_image[i][j];
			}
			if (lepton_image[i][j] < minval) {
				minval = lepton_image[i][j];
			}
		}
	}
	printf("maxval = %u\n",maxval);
	printf("minval = %u\n",minval);
	
	fprintf(f,"P2\n80 60\n%u\n",maxval-minval);
	for(i=0;i<60;i++)
	{
		for(j=0;j<80;j++)
		{
			fprintf(f,"%d ", lepton_image[i][j] - minval);
		}
		fprintf(f,"\n");
	}
	fprintf(f,"\n\n");

	fclose(f);
}

int transfer(int fd)
{
	int ret;
	int i;
	int frame_number;
	uint8_t tx[VOSPI_FRAME_SIZE] = {0, };
	struct spi_ioc_transfer tr = {
		.tx_buf = (unsigned long)tx,
		.rx_buf = (unsigned long)lepton_frame_packet,
		.len = VOSPI_FRAME_SIZE,
		.delay_usecs = delayu,
		.speed_hz = speed,
		.bits_per_word = bits,
	};

	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 1)
		pabort("can't send spi message");

	if(((lepton_frame_packet[0]&0xf) != 0x0f))
	{
		frame_number = lepton_frame_packet[1];

		if(frame_number < 60 )
		{
			for(i=0;i<80;i++)
			{
				lepton_image[frame_number][i] = (lepton_frame_packet[2*i+4] << 8 | lepton_frame_packet[2*i+5]);
			}
		}
	}
	return frame_number;
}

static void copy_image(void)
{
	int i;
	int j;
	for (i = 0; i<60; i++)
	{
		for (j = 0; j<80; j++)
		{
		  lepton_image_scale[i][j]  = lepton_image[i][j];
		}
	}
}

static void scale_image(void)
{
	int i;
	int j;
	int max = 0;
	int min = 0xffff;

	for (i = 0; i<60; i++)
	{
		for (j = 0; j<80; j++)
		{
			if (lepton_image_scale[i][j] > max)
			{
				max = lepton_image_scale[i][j];
			}

			if (lepton_image_scale[i][j] < min)
			{
				min = lepton_image_scale[i][j];
			}
		}
	}

	printf("min:%d max:%d\n\r",min,max);
    tempmin = min ;
    tempmax = max ;
	for (i = 0; i<60; i++)
	{
		for (j = 0; j<80; j++)
		{
			lepton_image_scale[i][j] -= min;
			lepton_image_scale[i][j] = ((lepton_image_scale[i][j] * 1280) / (max - min)) + 1;
		}
	}
}

static void generate_pallette(void)
{
    int i ;
	for (i = 0; i <= 255; i++)
	{
		// Red (255,0,0) to yellow (255,255,0)
		col[0][NCOLORS - i] = 255;
		col[1][NCOLORS - i] = i;
		col[2][NCOLORS - i] = 0;

		// Yellow to green (0,255,0)
		col[0][NCOLORS - (i+256)] = 255-1;
		col[1][NCOLORS - (i+256)] = 255;
		col[2][NCOLORS - (i+256)] = 0;

		// Green to cyan (0,255,255)
		col[0][NCOLORS - (i + 512)] = 0;
		col[1][NCOLORS - (i + 512)] = 255;
		col[2][NCOLORS - (i + 512)] = i;

		// cyan to blue (0,0,255)
		col[0][NCOLORS - (i + 768)] = 0;
		col[1][NCOLORS - (i + 768)] = 255 - i;
		col[2][NCOLORS - (i + 768)] = 255;

		//blue to black
		col[0][NCOLORS - (i + 1024)] = 0;
		col[1][NCOLORS - (i + 1024)] = 0;
		col[2][NCOLORS - (i + 1024)] = 255 - i;
	}
 	       
	col[0][0] = 0; //blue
	col[1][0] = 0; //green 
	col[2][0] = 0; //red

	for (i = 0; i < NCOLORS; i++)
	{
      colors[ i ]  =  ((col[0][i] & 0xF8 ) << 8) 
      	           |  ((col[1][i] & 0xFC ) << 3) 
      	           |  ((col[2][i] & 0xF8 ) >> 3) ;
    }
}

void LCD_PutPixel(unsigned int x, unsigned int y, unsigned int color)
{
  LCD_SetCursor(x,y);
  LCD_WriteIndex(0x22);
  LCD_WriteData(color);
}
 
static void DrawPixels()
{
	int step ;
	step = 3;
	int i,j;
	for (j = 0; j<80; j++)
	{
		for (i = 0; i<60; i++)
		{
		LCD_Rectangle(j*step,i*step,(j+1)*step,(i+1)*step,colors[lepton_image_scale[i][j]]);		
//		LCD_PutPixel(j, i, colors[lepton_image_scale[i][j]]);
		}
	}
}

static void initspi()
{
	int ret;
	
	fd = open(device, O_RDWR);
	if (fd < 0)
	{
		pabort("can't open device");
	}
	ret = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if (ret == -1)
	{
		pabort("can't set spi mode");
	}
	ret = ioctl(fd, SPI_IOC_RD_MODE, &mode);
	if (ret == -1)
	{
		pabort("can't get spi mode");
	}
	ret = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if (ret == -1)
	{
		pabort("can't set bits per word");
	}
	ret = ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits);
	if (ret == -1)
	{
		pabort("can't get bits per word");
	}
	ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if (ret == -1)
	{
		pabort("can't set max speed hz");
	}

	ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed);
	if (ret == -1)
	{
		pabort("can't get max speed hz");
	}
}

static void initi2c()
{
	fdi = wiringPiI2CSetup(ID);
	printf("I2C setup return : %d\n",fdi);
}

const unsigned char font[] = {
    0,   // format
    11,   // fontHeight
    9,   // ascent
    33,   // from
    95,   // count
    59,   // totalWidth/8
        0x00,0x01,0x00,0x04,0x00,0x0B,0x00,0x10,0x00,0x1A,0x00,0x21,0x00,0x22,0x00,0x25,0x00,0x28,0x00,0x2D,0x00,0x34,0x00,0x36,0x00,0x39,0x00,0x3A,0x00,0x3F,0x00,0x44,0x00,0x49,0x00,0x4E,0x00,0x53,0x00,0x59,0x00,0x5E,0x00,0x63,0x00,0x68,0x00,0x6D,0x00,0x72,0x00,0x73,0x00,0x75,0x00,0x7B,0x00,0x82,0x00,0x88,0x00,0x8C,0x00,0x94,0x00,0x9A,0x00,0xA0,0x00,0xA7,0x00,0xAE,0x00,0xB3,0x00,0xB8,0x00,0xBF,0x00,0xC5,0x00,0xC8,0x00,0xCC,0x00,0xD2,0x00,0xD7,0x00,0xDE,0x00,0xE4,0x00,0xEB,0x00,0xF0,0x00,0xF7,0x00,0xFD,0x01,0x03,0x01,0x0A,0x01,0x10,0x01,0x16,0x01,0x1F,0x01,0x25,0x01,0x2C,0x01,0x32,0x01,0x35,0x01,0x3A,0x01,0x3D,0x01,0x44,0x01,0x4B,0x01,0x4D,0x01,0x52,0x01,0x57,0x01,0x5C,0x01,0x61,0x01,0x66,0x01,0x6A,0x01,0x6F,0x01,0x74,0x01,0x75,0x01,0x78,0x01,0x7D,0x01,0x7E,0x01,0x87,0x01,0x8C,0x01,0x91,0x01,0x96,0x01,0x9B,0x01,0x9F,0x01,0xA3,0x01,0xA7,0x01,0xAC,0x01,0xB1,0x01,0xB8,0x01,0xBD,0x01,0xC2,0x01,0xC6,0x01,0xCB,0x01,0xCC,0x01,0xD1,0x01,0xD8,0x01,0xD8,    // Col ends
    0x50,0x04,0x00,0x00,0x4C,0x20,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x38,0x00,0x10,0x20,0x00,0x80,0xC1,0x09,0x84,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7C,0x00,    // Line 0
    0xD1,0x44,0x62,0x18,0x52,0xA8,0x00,0x02,0xE2,0x39,0xC1,0x7C,0xDF,0x73,0x80,0x00,0x00,0xE3,0xE3,0x3C,0x3D,0xF3,0xFF,0x3D,0x0F,0x78,0x61,0x8F,0x13,0x9E,0x39,0xE3,0xDF,0xE1,0x86,0x23,0x0C,0x1F,0xE4,0x08,0x80,0x08,0x20,0x00,0x81,0x01,0x00,0x84,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x92,0x00,    // Line 1
    0xD1,0x4F,0x92,0x24,0x52,0x70,0x80,0x05,0x1E,0x46,0x23,0x41,0x01,0x8C,0x40,0x00,0x00,0x14,0x13,0x22,0x43,0x0A,0x10,0x43,0x0A,0x18,0xA1,0x8F,0x14,0x51,0x45,0x14,0x22,0x21,0x86,0x23,0x0A,0x20,0x62,0x09,0x40,0x00,0x20,0x00,0x81,0x01,0x00,0x84,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x92,0x00,    // Line 2
    0x87,0xF4,0x94,0x24,0x21,0xA8,0x80,0x05,0x12,0x04,0x25,0x42,0x02,0x8C,0x68,0x60,0x30,0x19,0xD4,0xA2,0x81,0x06,0x10,0x81,0x0A,0x19,0x21,0x56,0x98,0x31,0x83,0x14,0x02,0x21,0x85,0x54,0x91,0x40,0xA2,0x0A,0x20,0x03,0xBC,0xE7,0xBB,0xDF,0xEB,0x8F,0xDD,0xE7,0x79,0xF6,0xFF,0x18,0xC9,0x8C,0x7C,0x92,0x00,    // Line 3
    0x82,0x94,0x64,0x19,0x21,0x20,0x80,0x09,0x12,0x08,0xC9,0x7B,0xC2,0x74,0x69,0x9F,0xCC,0x2A,0x54,0xBE,0x81,0x07,0xFE,0x81,0xFA,0x1A,0x21,0x56,0x98,0x31,0x83,0x13,0x02,0x21,0x49,0x54,0x60,0x81,0x21,0x0C,0x10,0x00,0x63,0x18,0xC5,0x23,0x19,0x96,0x23,0x18,0xC6,0x39,0x09,0x18,0xC9,0x52,0x84,0x92,0x31,    // Line 4
    0x82,0x8E,0x09,0xA5,0x21,0x07,0xF3,0x89,0x12,0x10,0x31,0x06,0x24,0x8B,0xC6,0x00,0x03,0x4A,0x54,0xA1,0x81,0x06,0x10,0x8F,0x0A,0x1E,0x21,0x26,0x58,0x3E,0x83,0xE0,0xC2,0x21,0x49,0x54,0x60,0x82,0x21,0x08,0x00,0x03,0xE3,0x08,0xFD,0x23,0x19,0xA6,0x23,0x18,0xC6,0x31,0x89,0x15,0x55,0x22,0x8B,0x11,0xC9,    // Line 5
    0x8F,0xC5,0x0A,0x62,0x21,0x00,0x80,0x11,0x12,0x20,0x3F,0x86,0x24,0x88,0x41,0x9F,0xCC,0x4A,0x5F,0xE1,0x81,0x06,0x10,0x83,0x0A,0x19,0x21,0x26,0x58,0x30,0x83,0x20,0x22,0x21,0x49,0x54,0x90,0x84,0x20,0x88,0x00,0x04,0x63,0x08,0xC1,0x23,0x19,0xE6,0x23,0x18,0xC6,0x30,0x69,0x15,0x55,0x22,0x90,0x92,0x46,    // Line 6
    0x05,0x05,0x12,0x63,0x21,0x00,0x84,0x51,0x12,0x42,0x21,0x46,0x28,0x88,0xA8,0x60,0x30,0x09,0xA8,0x61,0x43,0x0A,0x10,0x43,0x0A,0x18,0xA1,0x06,0x34,0x50,0x45,0x14,0x22,0x21,0x30,0x89,0x08,0x88,0x20,0x88,0x00,0x04,0x63,0x18,0xC5,0x23,0x19,0x96,0x23,0x18,0xC6,0x30,0x29,0x12,0x22,0x51,0x20,0x92,0x00,    // Line 7
    0x85,0x1E,0x11,0x9C,0x92,0x00,0x84,0x60,0xEF,0xFD,0xC1,0x39,0xC8,0x73,0x28,0x00,0x00,0x44,0x08,0x7E,0x3D,0xF3,0xF0,0x3D,0x0F,0xE8,0x7F,0x06,0x33,0x90,0x39,0x0B,0xC2,0x1E,0x30,0x89,0x08,0x8F,0xE0,0x48,0x00,0x03,0xFC,0xE7,0xB9,0x1F,0x19,0x8E,0x23,0x17,0x79,0xF1,0xC6,0xF2,0x22,0x89,0x3C,0x92,0x00,    // Line 8
    0x00,0x04,0x00,0x00,0x12,0x00,0x04,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x03,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x48,0x00,0x00,0x00,0x00,0x00,0x02,0x01,0x00,0x00,0x00,0x40,0x20,0x00,0x00,0x00,0x01,0x00,0x92,0x00,    // Line 9
    0x00,0x04,0x00,0x00,0x0C,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x38,0x0F,0xE0,0x00,0x00,0x00,0x1C,0x06,0x00,0x00,0x00,0x40,0x20,0x00,0x00,0x00,0x02,0x00,0x7C,0x00,    // Line 10
};


int LCD_DrawChar(unsigned char xx, unsigned int yy, int c, unsigned int color)
{

    if (c == 32)
        return font[1]>>2;  // Space is 1/4 font height (yuk);
    int i = c - font[3];
    if (i < 0 || i >= font[4]) return 0;
    int ci = 6 + i * 2;
    int width = (font[ci] << 8) | font[ci + 1];   // simplify
    int height = font[1];
    int src = 0;
    if (i > 0)
    {
        src = (font[ci - 2] << 8) | font[ci - 1];
        width -= src;
    }
   
    //  clip?
    src += (6 + 2 * font[4]) * 8;    // start of pixels (roll into cols)
    unsigned char mask = 0x80 >> (src & 7);  // Mask at start of line
    int end = (src+width) >> 3;     // number of unsigned char s read
    src >>= 3;
    int makeup = font[5] - (end + 1 - src);
     unsigned char y;
     unsigned char heightu;
     heightu = height ;
    for (y = 0; y < heightu; y++)
    {
        unsigned char p = font[src++];
        unsigned char m = mask;
        unsigned char x ;
        unsigned char widthu ;
        widthu = width ;
        for (x = 0; x < widthu; x++)
        {
            if (p & m)
                LCD_PutPixel(x + xx, y + yy, color);
            m >>= 1;
            if (m == 0)
            {
                p = font[src++];
                if (p == 0 && (width-x) <= 8)   // early out
                    break;
                m = 0x80;
            }
        }
        src += makeup;
    }
    return width;
}
void LCD_DrawString2(const char* s, int len, unsigned char x, unsigned int y, unsigned int color)
{
int i;
    for (i = 0; i < len; i++)
        x += LCD_DrawChar(x,y,s[i], color) + 1;
}
void LCD_DrawString(const char* s, unsigned char x, unsigned int y, unsigned int color)
{
    LCD_DrawString2(s,strlen(s),x,y, color);
}


//**********************************************************
// Main
//**********************************************************

int main(int argc, char *argv[])
{
	int ret = 0;
	
   	initspi();
    initi2c();	
   	initgpio();
    initlcd();
    initi2c();
	generate_pallette();
	LCD_Clear(0);
   	LCD_DrawString("Mim Temp [deg]:", 30, 200, BLUE);
    LCD_DrawString("Max Temp [deg]:", 30, 220, RED);
      
	printf("spi mode: %d\n", mode);
	printf("bits per word: %d\n", bits);
	printf("max speed: %d Hz (%d KHz)\n", speed, speed/1000);

	//loop
	while(1)
	{
//		lepton_info();

	while(transfer(fd)!=59){}
    	copy_image();
	    scale_image();
	    
	    tempminf = 0.03385*tempmin + T_C - 276.96 ;
	    tempmaxf = 0.03385*tempmax + T_C - 276.96 ;
	    
	    printf("min:%f max:%f\n\r",tempminf,tempmaxf);
	    
        sprintf(s1, "%f", tempminf);
	    sprintf(s2, "%f", tempmaxf);
	    
	    LCD_Rectangle(130,200,200,230,0);
	    LCD_DrawString(s1, 130, 200, BLUE);
        LCD_DrawString(s2, 130, 220, RED);
	    
    	DrawPixels();
 
 }
    
 //loop exit
 	
	close(fd);
    printf("fd close\n");

	save_pgm_file();

	return ret;
}
